import logging
import random

from torch.utils.data import Dataset

from .reader import VideoReader

logger = logging.getLogger('VideoDataset')


class VideoDataset(Dataset):
    """
    Reading the training Vimeo dataset
    key example: train/00001/0001/im1.png
    """

    def __init__(self, video_reader: VideoReader, num_frames=7, random_reverse=True):
        super(VideoDataset, self).__init__()
        self.reader = video_reader
        self.num_frames = num_frames

        # temporal augmentation
        self.random_reverse = random_reverse
        logger.info('Temporal augmentation with random reverse is {}.'.format(self.random_reverse))

        self.video_list = self.reader.list_videos()
        self.len = sum([len(video) // num_frames for video in self.video_list])

    def __len__(self):
        return self.len

    def __getitem__(self, index):
        frame_index = index * self.num_frames

        # Get frame list
        frames = []
        for i in range(len(self.video_list)):
            if 0 <= frame_index <= len(self.video_list[i]) - self.num_frames:
                image_paths = self.video_list[i][frame_index:frame_index + self.num_frames]
                logger.debug("Reading images: %s" % image_paths)
                frames.extend(self.reader.read_images(image_paths))
                break
            else:
                frame_index = frame_index - (len(self.video_list[i]) - len(self.video_list[i]) % self.num_frames)
        if self.random_reverse and random.random() < 0.5:
            frames.reverse()
        return frames



import abc
import numpy as np


class VideoReader(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def valid(self) -> bool:
        pass

    @abc.abstractmethod
    def read_images(self, paths: [str]) -> [np.ndarray]:
        pass

    @abc.abstractmethod
    def list_videos(self) -> [[str]]:
        pass


class VideoReaderList(VideoReader):
    """A series of VideoReader, different image output but same image name"""

    def __init__(self, *args: VideoReader):
        self.readers = args
        assert self.valid(), "Not a valid VideoReaderList"

    def valid(self):
        if len(self.readers):
            return True
        l0 = self.readers[0].list_videos()
        for reader in self.readers[1:]:
            if l0 != reader.list_videos():
                return False
        return True

    def list_videos(self) -> [[str]]:
        return self.readers[0].list_videos()

    def read_images(self, paths: [str]) -> [(np.ndarray,)]:
        images = list(zip(*(reader.read_images(paths) for reader in self.readers)))
        return images



import os

import cv2


class DirectoryImageReader:
    def __init__(self, root: str):
        """Read image file from directory
        :param root: root of the directory
        """
        super().__init__()
        self.root = os.path.expanduser(root)
        assert self.valid(), "Not a valid DirectoryImageReader"

    def valid(self) -> bool:
        return os.path.isdir(self.root)

    def read_image(self, path: str):
        """Read a image from the directory.
        :param path: path of the file in directory
        :returns array: (H, W, C) BGR image.
        """
        path = path[1:] if path[0] == "/" or path[0] == "\\" else path
        return cv2.imread(os.path.join(self.root, path), cv2.IMREAD_COLOR)

    def list_images(self, path: str = None) -> [str]:
        """List images in the dir.
        :param path: path of the dir
        :returns paths of the image files in the dir, include sub dir
        """
        if path is None or len(path) <= 0:
            root = self.root
        else:
            path = path[1:] if path[0] == "/" or path[0] == "\\" else path
            root = os.path.join(self.root, path)
        images = []
        for top, dirs, files in os.walk(root):
            for file in files:
                abspath = os.path.join(top, file)
                # if filetype.is_image(abspath): # should check, but cost too much time
                images.append(os.path.relpath(abspath, root))
        return images

    def is_dir(self, path: str):
        return os.path.isdir(os.path.join(self.root, path))



from .REDSZipReader import REDSZipReader


def REDSReader(path: str, root: str):
    """
    Construct a vimeo dataset.
    :param path: path to REDS dataset zip file.
    :param root: path to REDS dataset file in zip. its sub dir should be <video index>/<frame index>.png. e.g. "test/test_sharp_bicubic/X4" in test_sharp_bicubic.zip
    :return: a simpletorchvideo.reader.VideoReader
    """
    reader = None
    try:
        reader = REDSZipReader(path, root)
    except:
        pass
    assert reader is not None, "%s is not a valid REDS dataset" % path
    return reader



import os


def parse_video_list_from_image_list(image_list: [str]) -> [[str]]:
    video_last = ''
    video_list = []
    for path in sorted(image_list):
        video = os.path.dirname(os.path.dirname(path))
        if not video == video_last:
            video_list.append([])
            video_last = video
        video_list[-1].append(path)
    return video_list



import logging
import os
import zipfile
import cv2
import numpy as np

logger = logging.getLogger('base')


class ZipImageReader:
    def __init__(self, path: str):
        """Read data from zip
        :param path: path of the zip file
        """
        super().__init__()
        self.path = os.path.expanduser(path)
        assert self.valid(), "Not a valid ZipReader"
        self.file = None

    def valid(self) -> bool:
        return zipfile.is_zipfile(self.path)

    @staticmethod
    def _format_path(path):
        path = path.replace("\\", "/")
        path = path[1:] if path[0] == "/" else path
        path = path[0:-1] if path[-1] == "/" else path
        return path

    def read_image(self, path: str):
        """Read a file from a zip.
        :param path: path of the file in zip
        :returns buffer-like file content
        """
        path = self._format_path(path)
        try:
            img_bytes = self.file.read(path)
        except zipfile.BadZipFile:
            self.file.close()
            logger.debug("Reopen zip file: %s" % self.path)
            self.file = zipfile.ZipFile(self.path, "r")
            img_bytes = self.file.read(path)
        return cv2.imdecode(np.frombuffer(img_bytes, np.uint8), cv2.IMREAD_COLOR)

    def _prepare_zip(self):
        if self.file is None:
            self.file = zipfile.ZipFile(self.path, "r")

    def list_images(self, path: str) -> [str]:
        """List a dir zip.
        :param path: path of the dir in zip
        :returns paths of the files in the dir, include sub dir
        """
        path = self._format_path(path)
        self._prepare_zip()
        return sorted(list(filter(lambda f: f[-1] != "/" and f.startswith(path), self.file.namelist())))

    def getinfo(self, path: str):
        self._prepare_zip()
        return self.file.getinfo(path)



import numpy as np
from simpletorchvideo.reader import VideoReader, DirectoryImageReader
from .util import *


class VimeoDirReader(VideoReader):
    def __init__(self, dir_path: str, include_list: [str]):
        """Read a Vimeo dataset directory. structure like sequences/<video index>/<slice index>/<image name>.png
        :param dir_path: path to vimeo dataset directory.
        :param include_list: include of vimeo dataset. format like content in vimeo_septuplet/sep_trainlist.txt, <video index>/<slice index>
        """
        super().__init__()
        self.reader = DirectoryImageReader(os.path.join(dir_path, "sequences"))
        self.include_list = sorted(include_list)
        assert self.valid(), "Not a valid Vimeo directory"

    def valid(self) -> bool:
        if not self.reader.valid():
            return False
        try:
            for p in self.include_list:
                if not self.reader.is_dir(p):
                    return False
            return True
        except:
            return False

    def read_images(self, paths: [str]) -> [np.ndarray]:
        return [self.reader.read_image(path) for path in paths]

    def list_videos(self) -> [[str]]:
        return parse_video_list_from_image_list(self.reader.list_images())



from .VimeoDirReader import VimeoDirReader
from .VimeoZipReader import VimeoZipReader


def VimeoReader(path: str, include_list: [str]):
    """
    Construct a vimeo dataset.
    :param path: Could be path to a vimeo_septuplet.zip or a vimeo_septuplet directory
    if a vimeo_septuplet.zip, structure in zip should like vimeo_septuplet/sequences/<video index>/<slice index>/<image name>.png
    if a vimeo_septuplet directory structure in dir should  like sequences/<video index>/<slice index>/<image name>.png
    :param include_list: include of vimeo dataset. format like content in vimeo_septuplet/sep_trainlist.txt, <video index>/<slice index>
    :return: a simpletorchvideo.reader.VideoReader
    """
    reader = None
    try:
        reader = VimeoDirReader(path, include_list)
    except:
        pass
    try:
        reader = VimeoZipReader(path, include_list)
    except:
        pass
    assert reader is not None, "%s is not a valid vimeo dataset" % path
    return reader



import os
import numpy as np
from simpletorchvideo.reader.util import VideoReader
from .ffmpeg import read_video_meta, read_video_sequence


class FFmpegReader(VideoReader):
    def __init__(self, videos: [str]):
        """Read data with ffmpeg
        :param videos: paths of the videos
        """
        super().__init__()
        self.paths = sorted(os.path.expanduser(path) for path in videos)
        assert self.valid(), "Hot a valid FFmpegReader"

    def valid(self) -> bool:
        try:
            for path in self.paths:
                _, _, _ = read_video_meta(path)
            return True
        except:
            return False

    @staticmethod
    def split_video_path(path: str):
        """split the path like <video path>/<frame index>
        :param path: path of the frame. Format: <video path>/<frame index>. e.g. such as: folder/some-video.mp4/03
        :returns (<video path>, <frame index>)
        """
        path = os.path.expanduser(path)
        frame_pos = int(os.path.basename(path))
        video_path = os.path.dirname(path)
        return video_path, frame_pos

    def read_images(self, paths: [str]) -> [np.ndarray]:
        """Read images using ffmpeg from video file.
        :param paths: paths of the images. Format: <video path>/<frame index>. e.g. such as: folder/some-video.mp4/03
        :returns array: (H, W, C) BGR image.
        """
        if len(paths) == 0:
            return []
        video_path, start = self.split_video_path(paths[0])
        stop = start
        for path in paths[1:]:
            vpath, pos = self.split_video_path(path)
            assert vpath == video_path, "should not read from other video!"
            assert pos == stop + 1, "should be a sequence"
            stop = pos
        width, height, n_frames = read_video_meta(video_path)
        assert n_frames > stop, "max frame num is %d!" % n_frames
        frames = read_video_sequence(video_path, start, stop + 1, width, height)
        return [frames[i, :, :] for i in range(frames.shape[0])]

    def list_videos(self) -> [[str]]:
        paths = []
        for path in self.paths:
            _, _, n_frames = read_video_meta(path)
            paths.append([os.path.join(path, "%08d" % i) for i in range(n_frames)])
        return paths



#!/usr/bin/env python
# coding: utf-8

from setuptools import setup

with open("README.md", "r", encoding='utf8') as fh:
    long_description = fh.read()

package_dir = {
    'simpletorchvideo': 'simpletorchvideo',
    'simpletorchvideo.reader': 'simpletorchvideo/reader',
    'simpletorchvideo.ffmpeg': 'simpletorchvideo/ffmpeg',
    'simpletorchvideo.reds': 'simpletorchvideo/reds',
    'simpletorchvideo.vimeo': 'simpletorchvideo/vimeo',
}

setup(
    name='simpletorchvideo',
    version='0.1.0',
    author='yindaheng98',
    author_email='yindaheng98@163.com',
    url='https://github.com/yindaheng98/simpltorchvideo',
    description=u'Some useful video dataset in pytorch',
    long_description=long_description,
    long_description_content_type="text/markdown",
    package_dir=package_dir,
    packages=[key for key in package_dir],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'torch>=1.12.0',
        'opencv-python>=4.6.0.66',
        'ffmpeg-python>=0.2.0',
    ],
)



import numpy as np
from simpletorchvideo.reader import VideoReader, ZipImageReader
from .util import *


class REDSZipReader(VideoReader):
    def __init__(self, zip_path: str, root: str):
        """Read a Vimeo dataset zip file.
        :param zip_path: path to REDS dataset zip file.
        :param root: path to REDS dataset file in zip. its sub dir should be <video index>/<frame index>.png. e.g. "test/test_sharp_bicubic/X4" in test_sharp_bicubic.zip
        """
        super().__init__()
        self.reader = ZipImageReader(zip_path)
        self.root = self._format_path(root) + '/'
        assert self.valid(), "Not a valid REDS zip"

    def valid(self) -> bool:
        if not self.reader.valid():
            return False
        try:
            if self.reader.getinfo(self.root) is None:
                return False
            return True
        except:
            return False

    @staticmethod
    def _format_path(path):
        path = path.replace("\\", "/")
        path = path[1:] if path[0] == "/" else path
        path = path[0:-1] if path[-1] == "/" else path
        return path

    def read_images(self, paths: [str]) -> [np.ndarray]:
        return [self.reader.read_image(self.root + self._format_path(path)) for path in paths]

    def list_videos(self) -> [[str]]:
        return parse_video_list_from_image_list([p.replace(self.root, '') for p in self.reader.list_images(self.root)])



import os

import cv2
import ffmpeg
import numpy as np

loglevel = 'error'


def read_video_sequence(path: str, start: int, stop: int, width: int, height: int):
    path = os.path.expanduser(path)
    process = (ffmpeg
               .input(path, loglevel=loglevel)
               .trim(start_frame=start, end_frame=stop)
               .setpts('PTS-STARTPTS')
               .output('pipe:', format='rawvideo', pix_fmt='rgb24')
               .run_async(pipe_stdout=True, pipe_stderr=False))
    out, err = process.communicate()
    if err:
        raise err
    return np.frombuffer(out, np.uint8).reshape([-1, height, width, 3])


def read_video_meta(path: str):
    path = os.path.expanduser(path)
    capture = cv2.VideoCapture(path)
    n_frames = int(capture.get(cv2.CAP_PROP_FRAME_COUNT))
    width = int(capture.get(cv2.CAP_PROP_FRAME_WIDTH))
    height = int(capture.get(cv2.CAP_PROP_FRAME_HEIGHT))
    capture.release()
    return width, height, n_frames


if __name__ == "__main__":
    width, height, n_frames = read_video_meta("~/Videos/1080p.flv")
    print(width, height, n_frames)
    print(read_video_sequence("~/Videos/1080p.flv", 1012, 1080, width, height).shape)



import numpy as np
from simpletorchvideo.reader import VideoReader, ZipImageReader
from .util import *


class VimeoZipReader(VideoReader):
    def __init__(self, zip_path: str, include_list: [str]):
        """Read a Vimeo dataset zip file. structure like vimeo_septuplet/sequences/<video index>/<slice index>/<image name>.png
        :param zip_path: path to vimeo dataset zip file.
        :param include_list: include of vimeo dataset. format like content in vimeo_septuplet/sep_trainlist.txt, <video index>/<slice index>
        """
        super().__init__()
        self.reader = ZipImageReader(zip_path)
        self.include_list = sorted([self._format_path(p) for p in include_list])
        assert self.valid(), "Not a valid Vimeo zip"

    def valid(self) -> bool:
        if not self.reader.valid():
            return False
        try:
            if self.reader.getinfo('vimeo_septuplet/sequences/') is None:
                return False
            for p in self.include_list:
                if self.reader.getinfo('vimeo_septuplet/sequences/' + self._format_path(p) + '/') is None:
                    return False
            return True
        except:
            return False

    @staticmethod
    def _format_path(path):
        path = path.replace("\\", "/")
        path = path[1:] if path[0] == "/" else path
        path = path[0:-1] if path[-1] == "/" else path
        return path

    def read_images(self, paths: [str]) -> [np.ndarray]:
        return [self.reader.read_image('vimeo_septuplet/sequences/' + self._format_path(path)) for path in paths]

    def list_videos(self) -> [[str]]:
        paths = []
        root = "vimeo_septuplet/sequences/"
        for include in self.include_list:
            paths.extend([p.replace(root, '') for p in self.reader.list_images(root + include)])
        return parse_video_list_from_image_list(paths)
